//----------------------------------------------------------------------
//							4Bit Lcd Driver
//				   Write By Mohammad mazarei @ 1390/6/25
//					Email: mohammad.mazarei@gmail.com
//				Publish In Iranmicro.ir{Kavairelectronic}
//----------------------------------------------------------------------

#include "lcd.h"
#include "tdelay.h"
#include <at91sam7x256.h>
#include <lib_at91sam7x256.h>

#define LCD_RS_HI		AT91F_PIO_SetOutput(LCD_PORT,1<<LCD_RS)
#define LCD_RS_LO		AT91F_PIO_ClearOutput(LCD_PORT,1<<LCD_RS)

#define LCD_RW_HI		AT91F_PIO_SetOutput(LCD_PORT,1<<LCD_RW)
#define LCD_RW_LO		AT91F_PIO_ClearOutput(LCD_PORT,1<<LCD_RW)

#define LCD_EN_HI		AT91F_PIO_SetOutput(LCD_PORT,1<<LCD_EN)
#define LCD_EN_LO		AT91F_PIO_ClearOutput(LCD_PORT,1<<LCD_EN)

#define LCD_DATA_MASK	(((1<<LCD_4D)<<0)|((1<<LCD_4D)<<1)|((1<<LCD_4D)<<2)|((1<<LCD_4D)<<3))
#define LCD_OUT_MASK	((1<<LCD_RS)|(1<<LCD_RW)|(1<<LCD_EN)| LCD_DATA_MASK )

#define LCD_PUT_DATA(x)	AT91F_PIO_ClearOutput(LCD_PORT,LCD_DATA_MASK); AT91F_PIO_SetOutput(LCD_PORT,(x & 0x0F)<<LCD_4D)
#define LCD_PIN_DATA	(AT91F_PIO_GetInput(LCD_PORT)>>LCD_4D)&0x0F

#define LCD_LINE0_DDRAMADDR		0x00
#define LCD_LINE1_DDRAMADDR		0x40
#define LCD_LINE2_DDRAMADDR		0x14
#define LCD_LINE3_DDRAMADDR		0x54
#define LCD_DDRAM           7      // DB7: set DD RAM address
#define LCD_CGRAM           6      // DB6: set CG RAM address
#define LCD_CLR             0      // DB0: clear display
#define LCD_HOME            1      // DB1: return to home position

void LCD_STROBE(void)
{
LCD_EN_HI;
delay_us(10);
LCD_EN_LO;
}

void lcdBusyWait(void)
{
	// wait until LCD busy bit goes to zero
	// do a read from control register
	LCD_RS_LO;				// set RS to "control"
	AT91F_PIO_CfgInput(LCD_PORT,LCD_DATA_MASK);	// set data I/O lines to input (4bit)
	//outb(LCD_DATA_POUT, inb(LCD_DATA_POUT)|0xF0);	// set pull-ups to on (4bit)

	LCD_RW_HI;				// set R/W to "read"
	LCD_EN_HI;					// set "E" line
	LCD_Delay;				// wait
	while(LCD_PIN_DATA & 1<<3)
	{
		LCD_EN_LO;		// clear "E" line
		LCD_Delay;									// wait
		LCD_Delay;									// wait
		LCD_EN_HI;		// set "E" line
		LCD_Delay;									// wait
		LCD_Delay;									// wait
		
			LCD_EN_LO;	// clear "E" line
			LCD_Delay;								// wait
			LCD_Delay;								// wait
			LCD_EN_HI;	// set "E" line
			LCD_Delay;								// wait
			LCD_Delay;								// wait
		
	}
	LCD_EN_LO;			// clear "E" line
	
}


void lcd_write_byte (char Value)
{
	char upperNibble = Value >> 4;
	char lowerNibble = Value & 0x0F;

	LCD_PUT_DATA(upperNibble);
	LCD_STROBE();

	LCD_PUT_DATA(lowerNibble);
	LCD_STROBE();

}

void lcd_command(char cmd)
{
LCD_RS_LO;
LCD_Delay;
lcd_write_byte(cmd);
LCD_Delay; 
}
//*******************************************
void lcd_data(char ch)
{
	LCD_RS_HI;
	lcd_write_byte(ch);
	LCD_Delay;
}

void init_lcd(void){
 AT91C_BASE_PMC->PMC_PCER = ( 1<<LCD_PMC );
 AT91F_PIO_CfgOutput(LCD_PORT,LCD_OUT_MASK);
 AT91F_PIO_ClearOutput(LCD_PORT,LCD_OUT_MASK);


lcd_write_byte(0x30);
delay_us(10);
lcd_write_byte(0x30);
delay_us(5);
lcd_write_byte(0x30);
delay_us(5);
lcd_write_byte(0x20);
delay_us(5);

lcd_command(0x28);
lcd_command(0x0C);
lcd_command(0x08);
lcd_command(0x01);
lcd_command(0x02);
//lcd_command(0x06);
lcd_command(0x0c);
}

void lcd_gotoxy(char x, char y)
{
	register char DDRAMAddr;

	// remap lines into proper order
	switch(y)
	{
	case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
	case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
	case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
	case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
	default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}

	// set data address
	lcd_command(1<<LCD_DDRAM | DDRAMAddr);
}

void lcd_Home(void)
{
	// move cursor to home
	lcd_command(1<<LCD_HOME);
}

void lcd_cls(void)
{
	// clear LCD
	lcd_command(1<<LCD_CLR);
}

void lcd_LoadCustomChar(char* lcdCustomCharArray, char lcdCharNum)
{
	char i;

	// multiply the character index by 8
	lcdCharNum = (lcdCharNum<<3);	// each character occupies 8 bytes

	// copy the 8 bytes into CG (character generator) RAM
	for(i=0; i<8; i++)
	{
		// set CG RAM address
		lcd_command((1<<LCD_CGRAM) | (lcdCharNum+i));
		// write character data
		lcd_data(*(lcdCustomCharArray+i));
	}
	lcd_gotoxy(0,0);
}
void lcd_putc(char c){
   lcd_data(c);
}
void lcd_puts(char* data)
{
	while(*data!=0){
		lcd_data(*data++);
		}	
}
